<?php 
/* 

   You get the environment parameters associated with your Vitrium account
   For more details, refer to the following;
   https://enterprise.protectedpdf.com/secure/docapi

*/

require 'vendor/autoload.php';

use GuzzleHttp\Client;
use GuzzleHttp\Psr7\Request;
use GuzzleHttp\Psr7\Stream;
use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Promise\PromiseInterface;
$dotenv = Dotenv\Dotenv::createImmutable(__DIR__);
$dotenv->load();

function createGuid() {

  if (function_exists('com_create_guid'))
      return trim(com_create_guid(), '{}');

  return sprintf('%04X%04X-%04X-%04X-%04X-%04X%04X%04X', mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(16384, 20479), mt_rand(32768, 49151), mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(0, 65535));

}


function api_request($method, $apiEndPoint, $params = [], $data = [], $formData = [], $queryString = [], $type = null) {

    try {

    // Define default headers based on the type
    if ($type === null) {
        $defaultHeaders = [
            'Accept'       => 'application/json', 
            'Content-Type' => 'application/json'
        ];
    } else {
        $defaultHeaders = [
            'Content-Type' => 'multipart/form-data'
        ];
    }

    // Merge default headers with custom headers
    $headers = array_merge($defaultHeaders, $params);

    // Prepare the request body
    if ($type == 'multipart') {
        $formParams = formatMultipart($formData);
        $body = null;
    } else {
        $formParams = null;
        $body = json_encode($data);
    }

    // Create the HTTP client
    $client = new GuzzleHttp\Client([
        'verify'       => false,
        'query'        => $queryString,  
        'multipart'    => $formParams,
        'base_uri'     => $_ENV['VIT_SERVER_URL'],
        'headers'      => $headers,
        'body'         => $body,
        'timeout'      => 120 // if the file upload doesn't complete within this time it will throw exception error, change this value accordingly
    ]);

    // Send the request
	$r = $client->request($method, $apiEndPoint);
	$response = $r->getBody()->getContents();
    if($response){
        return $response;
    }


    } catch (GuzzleHttp\Exception\RequestException $e) {
        if ($e->hasResponse()) {
            $statusCode = $e->getResponse()->getStatusCode();
            $errorMessage = $e->getResponse()->getBody()->getContents();
        } else {
            $statusCode = 0;
            $errorMessage = $e->getMessage();
        }

        echo $errorMessage;
    }
}

function formatMultipart($dataArr) {
    $multipart = [];

    foreach ($dataArr as $key => $value) {
        $multipart[] = [
            'name' => $key,
            'contents' => $value,
        ];
    }

    return $multipart;
}

/* # Handshake. Using POST METHOD. 

   1. login/challenge requires "ClientNonce" which can be obtained by running createGuid() function.
   2. login/challenge returns "ServerNonce" which we need it to generate "ClientHash"
   3. login/response requires "ClientNonce", "ClientHash", "UserName", and [serverNonce + guid + password] to generate "ClientHash" 

*/

if(isset($_POST['submit'])){

	/* ################## HANDSHAKE START ################## */

// RETRIVE $clientHash FROM LOGIN/CHALLENGE => GUID -> ServerNonce -> ClientHash
$guid = createGuid(); // GENERATE GUID
    $response = api_request('POST', 'login/challenge', array(), array("ClientNonce" => $guid), array(), array(), null);
	$serverNonce = json_decode($response, true)['ServerNonce'];
	$userName    = $_ENV['VIT_USER_NAME'];
	$password    = $_ENV['VIT_USER_PASSWORD'];
	$clientHash  = strtoupper(hash_hmac('sha1', strtolower($guid) . strtolower($serverNonce) . $password, $password));

//RETRIVE $sessionToken & $accountToken FROM LOGIN/RESPONSE => GUID + ClientHash + UserName -> SessionToken & AccountToken -> AuthToken
    $response = api_request('POST', 'login/response', array(), array("ClientNonce" => $guid, "ClientHash" => $clientHash, "UserName" => $userName), array(), array(), null);
	$sessionToken = json_decode($response, true)['ApiSession']['Token'];
	$accountToken = json_decode($response, true)['Accounts'][0]['Id']; // There are multiple account associated and I am using INDEX 0 to access specific account.
  	$authToken    = array("X-VITR-ACCOUNT-TOKEN" => $accountToken, "X-VITR-SESSION-TOKEN" => $sessionToken);

    /* ################## HANDSHAKE END ################## */

if ($_FILES['file']['error'] === UPLOAD_ERR_OK) {

    $uploadDir = './uploads/';
    $fileName = $_FILES['file']['name'];
    $tempFilePath = $_FILES['file']['tmp_name'];

    // Chunk size in bytes (e.g., 4 MB)
    $chunkSize   = 1024 * 1024 * 4;
    $fileHandle  = fopen($tempFilePath, 'rb');
    $chunkIndex  = 1;
    $fileSize    = filesize($tempFilePath);
    $packetCount = ceil($fileSize / $chunkSize);


    /* initial post request with original file name, file size, packet count to obtain guid from response */
    $formData = [
        'ufname'      =>  $fileName,
        'umime'       => 'application/pdf',
        'usize'       =>  $fileSize,
        'packetcount' => $packetCount
    ];
    $response = api_request('POST', 'File', $authToken, array(), $formData, array(), 'multipart');

    $chunkGuid = json_decode($response, true)['Guid']; // need this guid to upload chunks to server

    while (!feof($fileHandle)) {

        /* create chunk file in upload directory */
        $chunk = fread($fileHandle, $chunkSize);
        $chunkFileName = $uploadDir . $fileName . '.part' . $chunkIndex;
        $chunkHandle = fopen($chunkFileName, 'wb');
        fwrite($chunkHandle, $chunk);
        fclose($chunkHandle);

        /* packet post request with chunk file, guid, packet index, total packet count to upload chunks to server */
        $stream = new Stream(fopen($chunkFileName, 'r'));
        $formData = [
            'file' => $stream
        ];
        $queryString = [
            'guid' =>  $chunkGuid,
            'packet'  => $chunkIndex,
            'total'  =>  $packetCount,
        ];
        api_request('POST', 'File', $authToken, array(), $formData, $queryString, 'multipart');

        $chunkIndex++;

    }



        /* doc post request to combine the chunks and create content */
        $dirFileName =  $chunkGuid.'.pdf'; // File Name must persist the format of $chunkGuid from above + file extension
        $formData = [
            'filename' => $dirFileName,
            'settings' => '{"Id":"","FolderId":"","PolicyId":"","FileName":"'.$fileName.'","Title":"'.$fileName.'","SkipPages":"","ExternalKey":"","VersionName":"Version: 1","EnableVersionNotification":true}'
        ];
        api_request('POST', 'doc', $authToken, array(), $formData, array(), 'multipart');

        // Clean up the individual chunk files (if desired)
        for ($i = 1; $i < $chunkIndex; $i++) {
            $chunkFileName = $uploadDir . $fileName . '.part' . $i;
            unlink($chunkFileName);
        }

        echo '<br>Task has been completed - '.$packetCount.' packets uploaded';

} else {

	echo "<script>alert('ERROR: Upload Failed');</script>";

}

}

/* 
	HTML FRONT END 
*/

?>

<form method="POST" enctype="multipart/form-data">
	<input type="file" name="file">
	<button type="submit" name="submit" multiple>Submit</button>
</form>


